
.. DO NOT EDIT.
.. THIS FILE WAS AUTOMATICALLY GENERATED BY SPHINX-GALLERY.
.. TO MAKE CHANGES, EDIT THE SOURCE PYTHON FILE:
.. "_as_gen/sph/run_sph_basic_disc.py"
.. LINE NUMBERS ARE GIVEN BELOW.

.. only:: html

    .. note::
        :class: sphx-glr-download-link-note

        :ref:`Go to the end <sphx_glr_download__as_gen_sph_run_sph_basic_disc.py>`
        to download the full example code.

.. rst-class:: sphx-glr-example-title

.. _sphx_glr__as_gen_sph_run_sph_basic_disc.py:


Basic disc simulation
========================

This simple example shows how to run a basic disc simulation in SPH

.. GENERATED FROM PYTHON SOURCE LINES 7-17

.. code-block:: Python
   :lineno-start: 9



    import shamrock

    # If we use the shamrock executable to run this script instead of the python interpreter,
    # we should not initialize the system as the shamrock executable needs to handle specific MPI logic
    if not shamrock.sys.is_initialized():
        shamrock.change_loglevel(1)
        shamrock.sys.init("0:0")








.. GENERATED FROM PYTHON SOURCE LINES 19-20

Setup units

.. GENERATED FROM PYTHON SOURCE LINES 20-32

.. code-block:: Python
   :lineno-start: 21


    si = shamrock.UnitSystem()
    sicte = shamrock.Constants(si)
    codeu = shamrock.UnitSystem(
        unit_time=3600 * 24 * 365,
        unit_length=sicte.au(),
        unit_mass=sicte.sol_mass(),
    )
    ucte = shamrock.Constants(codeu)
    G = ucte.G()









.. GENERATED FROM PYTHON SOURCE LINES 33-34

List parameters

.. GENERATED FROM PYTHON SOURCE LINES 34-80

.. code-block:: Python
   :lineno-start: 35


    # Resolution
    Npart = 1000000

    # Sink parameters
    center_mass = 1.0
    center_racc = 0.1

    # Disc parameter
    disc_mass = 0.01  # sol mass
    rout = 10.0  # au
    rin = 1.0  # au
    H_r_0 = 0.05
    q = 0.5
    p = 3.0 / 2.0
    r0 = 1.0

    # Viscosity parameter
    alpha_AV = 1.0e-3 / 0.08
    alpha_u = 1.0
    beta_AV = 2.0

    # Integrator parameters
    C_cour = 0.3
    C_force = 0.25


    # Disc profiles
    def sigma_profile(r):
        sigma_0 = 1.0  # We do not care as it will be renormalized
        return sigma_0 * (r / r0) ** (-p)


    def kep_profile(r):
        return (G * center_mass / r) ** 0.5


    def omega_k(r):
        return kep_profile(r) / r


    def cs_profile(r):
        cs_in = (H_r_0 * r0) * omega_k(r0)
        return ((r / r0) ** (-q)) * cs_in









.. GENERATED FROM PYTHON SOURCE LINES 81-82

Utility functions and quantities deduced from the base one

.. GENERATED FROM PYTHON SOURCE LINES 82-102

.. code-block:: Python
   :lineno-start: 83


    # Deduced quantities
    pmass = disc_mass / Npart
    bmin = (-rout * 2, -rout * 2, -rout * 2)
    bmax = (rout * 2, rout * 2, rout * 2)

    cs0 = cs_profile(r0)


    def rot_profile(r):
        return ((kep_profile(r) ** 2) - (2 * p + q) * cs_profile(r) ** 2) ** 0.5


    def H_profile(r):
        H = cs_profile(r) / omega_k(r)
        # fact = (2.**0.5) * 3. # factor taken from phantom, to fasten thermalizing
        fact = 1.0
        return fact * H









.. GENERATED FROM PYTHON SOURCE LINES 103-106

Start the context
The context holds the data of the code
We then init the layout of the field (e.g. the list of fields used by the solver)

.. GENERATED FROM PYTHON SOURCE LINES 106-110

.. code-block:: Python
   :lineno-start: 107


    ctx = shamrock.Context()
    ctx.pdata_layout_new()








.. GENERATED FROM PYTHON SOURCE LINES 111-112

Attach a SPH model to the data and configure it

.. GENERATED FROM PYTHON SOURCE LINES 112-141

.. code-block:: Python
   :lineno-start: 113


    model = shamrock.get_Model_SPH(context=ctx, vector_type="f64_3", sph_kernel="M4")

    # Generate the default config
    cfg = model.gen_default_config()
    # Use disc alpha model viscosity
    cfg.set_artif_viscosity_ConstantDisc(alpha_u=alpha_u, alpha_AV=alpha_AV, beta_AV=beta_AV)
    # use the Lodato Price 2007 equation of state
    cfg.set_eos_locally_isothermalLP07(cs0=cs0, q=q, r0=r0)
    # Use the given code units
    cfg.set_units(codeu)
    # Change particle mass
    cfg.set_particle_mass(pmass)
    # Set the CFL
    cfg.set_cfl_cour(C_cour)
    cfg.set_cfl_force(C_force)

    # Set the solver config to be the one stored in cfg
    model.set_solver_config(cfg)

    # Print the solver config
    model.get_current_config().print_status()

    # We want the patches to split above 10^8 part and merge if smaller than 1 part (e.g. disable patch)
    model.init_scheduler(int(1e8), 1)

    # Set the simulation box size
    model.resize_simulation_box(bmin, bmax)





.. rst-class:: sphx-glr-script-out

 .. code-block:: none

    ----- SPH Solver configuration ----- 
    units :  
    unit_length      : 149597870700 
    unit_mass        : 1.98847e+30 
    unit_current     : 1 
    unit_temperature : 1 
    unit_qte         : 1 
    unit_lumint      : 1 
    part mass 1e-08 ( can be changed using .set_part_mass() ) 
    cfl force 0.25 
    cfl courant 0.3 
    --- artificial viscosity config 
      Config Type : constant disc 
      alpha_AV   = 0.0125 
      alpha_u     = 1 
      beta_AV     = 2 
    --- artificial viscosity config (deduced) 
    ------------- 
    EOS config f64_3 : 
    locally isothermal (Lodato Price 2007) :  
    --- Bondaries config 
      Config Type : Free boundaries 
    --- Bondaries config config (deduced) 
    ------------- 
    ------------------------------------ 




.. GENERATED FROM PYTHON SOURCE LINES 142-143

Add the sink particle

.. GENERATED FROM PYTHON SOURCE LINES 143-147

.. code-block:: Python
   :lineno-start: 144


    # null position and velocity
    model.add_sink(center_mass, (0, 0, 0), (0, 0, 0), center_racc)








.. GENERATED FROM PYTHON SOURCE LINES 148-149

Create the setup

.. GENERATED FROM PYTHON SOURCE LINES 149-166

.. code-block:: Python
   :lineno-start: 150


    setup = model.get_setup()
    gen_disc = setup.make_generator_disc_mc(
        part_mass=pmass,
        disc_mass=disc_mass,
        r_in=rin,
        r_out=rout,
        sigma_profile=sigma_profile,
        H_profile=H_profile,
        rot_profile=rot_profile,
        cs_profile=cs_profile,
        random_seed=666,
    )

    # Print the dot graph of the setup
    print(gen_disc.get_dot())





.. rst-class:: sphx-glr-script-out

 .. code-block:: none

    digraph G {
    rankdir=LR;
    node_0 [label="GeneratorMCDisc"];
    node_2 [label="Simulation"];
    node_0 -> node_2;
    }





.. GENERATED FROM PYTHON SOURCE LINES 167-168

Apply the setup

.. GENERATED FROM PYTHON SOURCE LINES 168-170

.. code-block:: Python
   :lineno-start: 168

    setup.apply_setup(gen_disc)





.. rst-class:: sphx-glr-script-out

 .. code-block:: none

    Info: pushing data in scheduler, N = 1000000                          [DataInserterUtility][rank=0] 
    Info: reattributing data ...                                          [DataInserterUtility][rank=0] 
    Info: reattributing data done in  52.04 ms                            [DataInserterUtility][rank=0] 
    Info: Compute load ...                                                [DataInserterUtility][rank=0] 
    Info: run scheduler step ...                                          [DataInserterUtility][rank=0] 
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 9.60 us    (15.6%)  
    Info: summary :                                                               [LoadBalance][rank=0] 
    Info:  - strategy "psweep" : max = 1000000 min = 1000000                      [LoadBalance][rank=0] 
    Info:  - strategy "round robin" : max = 1000000 min = 1000000                 [LoadBalance][rank=0] 
    Info: Loadbalance stats :                                                     [LoadBalance][rank=0]
        npatch = 1
        min = 1000000
        max = 1000000
        avg = 1000000
        efficiency = 100.00%  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 2.42 us    (0.4%)
       patch tree reduce : 1163.00 ns (0.2%)
       gen split merge   : 781.00 ns  (0.1%)
       split / merge op  : 0/0
       apply split merge : 932.00 ns  (0.2%)
       LB compute        : 525.55 us  (97.6%)
       LB move op cnt    : 0
       LB apply          : 3.62 us    (0.7%)  
    Info: the setup took : 2.916227653 s                                            [SPH setup][rank=0] 




.. GENERATED FROM PYTHON SOURCE LINES 171-178

Run a single step to init the integrator and smoothing length of the particles
Here the htolerance is the maximum factor of evolution of the smoothing length in each
Smoothing length iterations, increasing it affect the performance negatively but increse the
convergence rate of the smoothing length
this is why we increase it temporely to 1.3 before lowering it back to 1.1 (default value)
Note that both ``change_htolerance`` can be removed and it will work the same but would converge
more slowly at the first timestep

.. GENERATED FROM PYTHON SOURCE LINES 178-183

.. code-block:: Python
   :lineno-start: 179


    model.change_htolerance(1.3)
    model.timestep()
    model.change_htolerance(1.1)





.. rst-class:: sphx-glr-script-out

 .. code-block:: none

    ---------------- t = 0, dt = 0 ---------------- 
    Info: summary :                                                               [LoadBalance][rank=0] 
    Info:  - strategy "psweep" : max = 1000000 min = 1000000                      [LoadBalance][rank=0] 
    Info:  - strategy "round robin" : max = 1000000 min = 1000000                 [LoadBalance][rank=0] 
    Info: Loadbalance stats :                                                     [LoadBalance][rank=0]
        npatch = 1
        min = 1000000
        max = 1000000
        avg = 1000000
        efficiency = 100.00%  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 26.56 us   (2.2%)
       patch tree reduce : 2.45 us    (0.2%)
       gen split merge   : 781.00 ns  (0.1%)
       split / merge op  : 0/0
       apply split merge : 1062.00 ns (0.1%)
       LB compute        : 1130.49 us (94.0%)
       LB move op cnt    : 0
       LB apply          : 3.51 us    (0.3%)  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 2.89 us    (71.3%)  
    Info: free boundaries skipping geometry update                            [PositionUpdated][rank=0] 
    Warning: smoothing length is not converged, rerunning the iterator ...    [Smoothinglength][rank=0]
         largest h = 0.30782750590945696 unconverged cnt = 999999   
    Warning: smoothing length is not converged, rerunning the iterator ...    [Smoothinglength][rank=0]
         largest h = 0.3566642885216895 unconverged cnt = 999999   
    Warning: smoothing length is not converged, rerunning the iterator ...    [Smoothinglength][rank=0]
         largest h = 0.4636635750781964 unconverged cnt = 999999   
    Warning: smoothing length is not converged, rerunning the iterator ...    [Smoothinglength][rank=0]
         largest h = 0.6027626476016553 unconverged cnt = 999997   
    Warning: smoothing length is not converged, rerunning the iterator ...    [Smoothinglength][rank=0]
         largest h = 0.7251187699241742 unconverged cnt = 999987   
    Warning: smoothing length is not converged, rerunning the iterator ...    [Smoothinglength][rank=0]
         largest h = 0.7333109890812106 unconverged cnt = 999937   
    Warning: smoothing length is not converged, rerunning the iterator ...    [Smoothinglength][rank=0]
         largest h = 0.7529481567236113 unconverged cnt = 999775   
    Warning: smoothing length is not converged, rerunning the iterator ...    [Smoothinglength][rank=0]
         largest h = 0.7529481567236118 unconverged cnt = 998210   
    Warning: smoothing length is not converged, rerunning the iterator ...    [Smoothinglength][rank=0]
         largest h = 0.7529481567236117 unconverged cnt = 938876   
    Warning: smoothing length is not converged, rerunning the iterator ...    [Smoothinglength][rank=0]
         largest h = 0.7529481567236117 unconverged cnt = 468324   
    Warning: smoothing length is not converged, rerunning the iterator ...    [Smoothinglength][rank=0]
         largest h = 0.7529481567236117 unconverged cnt = 10415   
    Warning: smoothing length is not converged, rerunning the iterator ...    [Smoothinglength][rank=0]
         largest h = 0.7529481567236117 unconverged cnt = 17   
    Info: smoothing length iteration converged                                [Smoothinglength][rank=0]
      eps min = 0, max = 9.350670822948002e-07
      iterations = 3  
    Info: conservation infos :                                                     [sph::Model][rank=0]
        sum v = (9.877002876192637e-06,-3.3153642116852934e-05,0)
        sum a = (-5.304120315706429e-18,2.846030702774449e-19,-1.2705494208814505e-21)
        sum e = 0.05004044613227272
        sum de = 6.4983308922165854e-21  
    Info: cfl dt = 3.907389496802996e-05 cfl multiplier : 0.01                     [sph::Model][rank=0] 
    Info: processing rate infos :                                                  [sph::Model][rank=0]
    ---------------------------------------------------------------------------------------
    | rank |  rate  (N.s^-1)  |     Nobj    | t compute (s) |  MPI   | alloc |  mem (max) |
    ---------------------------------------------------------------------------------------
    | 0    |    3.1681e+04    |     1000000 |   3.156e+01   |    0 % |   0 % |  950.83 MB |
    ---------------------------------------------------------------------------------------  
    Info: estimated rate : 0 (tsim/hr)                                             [sph::Model][rank=0] 




.. GENERATED FROM PYTHON SOURCE LINES 184-187

Manipulating the simulation
------------------------
Dump files (path relative to where you have started shamrock)

.. GENERATED FROM PYTHON SOURCE LINES 187-203

.. code-block:: Python
   :lineno-start: 188


    dump_folder = "_to_trash"
    import os

    os.system("mkdir -p " + dump_folder)

    # VTK dump
    model.do_vtk_dump(dump_folder + "/init_disc.vtk", True)

    # Shamrock restart dump files
    model.dump(dump_folder + "/init_disc.sham")

    # Phantom dump
    dump = model.make_phantom_dump()
    dump.save_dump(dump_folder + "/init_disc.phdump")





.. rst-class:: sphx-glr-script-out

 .. code-block:: none

    Info: dump to _to_trash/init_disc.vtk                                            [VTK Dump][rank=0]
                  - took 143.26 ms, bandwidth = 372.79 MB/s  
    Info: Dumping state to _to_trash/init_disc.sham                                       [SPH][rank=0] 
    Info: dump to _to_trash/init_disc.sham                                      [Shamrock Dump][rank=0]
                  - took 127.13 ms, bandwidth = 900.21 MB/s  




.. GENERATED FROM PYTHON SOURCE LINES 204-205

Single timestep

.. GENERATED FROM PYTHON SOURCE LINES 205-207

.. code-block:: Python
   :lineno-start: 205

    model.evolve_once()





.. rst-class:: sphx-glr-script-out

 .. code-block:: none

    ---------------- t = 0, dt = 3.907389496802996e-05 ---------------- 
    Info: summary :                                                               [LoadBalance][rank=0] 
    Info:  - strategy "psweep" : max = 1000000 min = 1000000                      [LoadBalance][rank=0] 
    Info:  - strategy "round robin" : max = 1000000 min = 1000000                 [LoadBalance][rank=0] 
    Info: Loadbalance stats :                                                     [LoadBalance][rank=0]
        npatch = 1
        min = 1000000
        max = 1000000
        avg = 1000000
        efficiency = 100.00%  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 17.28 us   (1.9%)
       patch tree reduce : 3.29 us    (0.4%)
       gen split merge   : 511.00 ns  (0.1%)
       split / merge op  : 0/0
       apply split merge : 942.00 ns  (0.1%)
       LB compute        : 838.88 us  (92.3%)
       LB move op cnt    : 0
       LB apply          : 13.74 us   (1.5%)  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 5.66 us    (77.7%)  
    Info: free boundaries skipping geometry update                            [PositionUpdated][rank=0] 
    Info: smoothing length iteration converged                                [Smoothinglength][rank=0]
      eps min = 1.0009746271268322e-14, max = 9.999963360720819e-07
      iterations = 1  
    Info: conservation infos :                                                     [sph::Model][rank=0]
        sum v = (9.877275295883587e-06,-3.315171760280455e-05,4.520315299363933e-11)
        sum a = (-1.5246593050577406e-18,1.0706496453294356e-18,-3.006966962752766e-20)
        sum e = 0.05004044658210748
        sum de = 1.6623973450838555e-07  
    Info: cfl dt = 0.0013286849078749257 cfl multiplier : 0.34                     [sph::Model][rank=0] 
    Info: processing rate infos :                                                  [sph::Model][rank=0]
    ---------------------------------------------------------------------------------------
    | rank |  rate  (N.s^-1)  |     Nobj    | t compute (s) |  MPI   | alloc |  mem (max) |
    ---------------------------------------------------------------------------------------
    | 0    |    1.6863e+05    |     1000000 |   5.930e+00   |    0 % |   0 % |  950.83 MB |
    ---------------------------------------------------------------------------------------  
    Info: estimated rate : 0.023720738077820318 (tsim/hr)                          [sph::Model][rank=0] 
    Info: iteration since start : 2                                                       [SPH][rank=0] 
    Info: time since start : 70.50197386100001 (s)                                        [SPH][rank=0] 




.. GENERATED FROM PYTHON SOURCE LINES 208-209

Evolve until a given time (code units)

.. GENERATED FROM PYTHON SOURCE LINES 209-211

.. code-block:: Python
   :lineno-start: 209

    model.evolve_until(0.001)





.. rst-class:: sphx-glr-script-out

 .. code-block:: none

    ---------------- t = 3.907389496802996e-05, dt = 0.0009609261050319701 ---------------- 
    Info: summary :                                                               [LoadBalance][rank=0] 
    Info:  - strategy "psweep" : max = 1000000 min = 1000000                      [LoadBalance][rank=0] 
    Info:  - strategy "round robin" : max = 1000000 min = 1000000                 [LoadBalance][rank=0] 
    Info: Loadbalance stats :                                                     [LoadBalance][rank=0]
        npatch = 1
        min = 1000000
        max = 1000000
        avg = 1000000
        efficiency = 100.00%  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 16.20 us   (1.7%)
       patch tree reduce : 6.59 us    (0.7%)
       gen split merge   : 501.00 ns  (0.1%)
       split / merge op  : 0/0
       apply split merge : 992.00 ns  (0.1%)
       LB compute        : 865.59 us  (93.2%)
       LB move op cnt    : 0
       LB apply          : 12.87 us   (1.4%)  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 4.58 us    (78.8%)  
    Info: free boundaries skipping geometry update                            [PositionUpdated][rank=0] 
    Info: smoothing length iteration converged                                [Smoothinglength][rank=0]
      eps min = 3.046203435208697e-14, max = 9.999794806296945e-07
      iterations = 2  
    Info: conservation infos :                                                     [sph::Model][rank=0]
        sum v = (9.883694937533913e-06,-3.310631640840492e-05,1.111660091448701e-09)
        sum a = (8.555032767268433e-19,1.0842021724855044e-19,-3.3881317890172014e-21)
        sum e = 0.050040718349661346
        sum de = 4.250271582630976e-06  
    Info: cfl dt = 0.002195483366595384 cfl multiplier : 0.56                      [sph::Model][rank=0] 
    Info: processing rate infos :                                                  [sph::Model][rank=0]
    ---------------------------------------------------------------------------------------
    | rank |  rate  (N.s^-1)  |     Nobj    | t compute (s) |  MPI   | alloc |  mem (max) |
    ---------------------------------------------------------------------------------------
    | 0    |    1.6435e+05    |     1000000 |   6.085e+00   |    0 % |   0 % |  950.83 MB |
    ---------------------------------------------------------------------------------------  
    Info: estimated rate : 0.5685264475734894 (tsim/hr)                            [sph::Model][rank=0] 
    Info: iteration since start : 3                                                       [SPH][rank=0] 
    Info: time since start : 76.824594761 (s)                                             [SPH][rank=0] 

    True



.. GENERATED FROM PYTHON SOURCE LINES 212-213

Get the sinks positions

.. GENERATED FROM PYTHON SOURCE LINES 213-215

.. code-block:: Python
   :lineno-start: 213

    print(model.get_sinks())





.. rst-class:: sphx-glr-script-out

 .. code-block:: none

    [{'pos': (-6.96448103523327e-12, -4.925022249208709e-11, -1.1568632444423742e-12), 'velocity': (-1.374628968486744e-08, -9.842692125776788e-08, -2.3137250214540352e-09), 'sph_acceleration': (-1.35487859454304e-05, -9.834742799767654e-05, -2.313723434691236e-06), 'ext_acceleration': (0.0, 0.0, 0.0), 'mass': 1.0, 'angular_momentum': (0.0, 0.0, 0.0), 'accretion_radius': 0.1}]




.. GENERATED FROM PYTHON SOURCE LINES 216-218

Get the fields as python dictionary of numpy arrays


.. GENERATED FROM PYTHON SOURCE LINES 220-223

.. warning::
    Do not do this on a large distributed simulation as this gather all data on MPI rank 0
    and will use a lot of memory (and crash if the simulation is too large)

.. GENERATED FROM PYTHON SOURCE LINES 223-225

.. code-block:: Python
   :lineno-start: 223

    print(ctx.collect_data())





.. rst-class:: sphx-glr-script-out

 .. code-block:: none

    adding ->  xyz
    adding ->  vxyz
    adding ->  axyz
    adding ->  axyz_ext
    adding ->  hpart
    adding ->  uint
    adding ->  duint
    {'xyz': array([[-7.65869271, -6.39642067, -0.95720353],
           [-7.55994437, -6.50503034, -1.17167654],
           [-7.51907833, -6.42069176, -1.15259277],
           ...,
           [ 7.55442992,  6.37259754,  1.01837309],
           [ 7.57542146,  6.28326004,  1.24365905],
           [ 7.65988183,  6.31067646,  0.96346708]], shape=(1000000, 3)), 'vxyz': array([[ 1.26848748e+00, -1.51895096e+00,  3.12306504e-05],
           [ 1.29103469e+00, -1.50060221e+00, -1.67518492e-05],
           [ 1.29096064e+00, -1.51196362e+00,  1.36791610e-05],
           ...,
           [-1.28208848e+00,  1.51994964e+00, -1.62093533e-07],
           [-1.27207321e+00,  1.53374707e+00,  1.55803842e-05],
           [-1.26170408e+00,  1.53154863e+00,  1.11059041e-05]],
          shape=(1000000, 3)), 'axyz': array([[ 2.20260256e-01,  2.08844814e-01,  3.12257824e-02],
           [ 2.35952850e-01,  1.26429677e-01, -1.67494312e-02],
           [ 2.61260995e-01,  1.51087699e-01,  1.36744045e-02],
           ...,
           [-2.50475689e-01, -2.32575157e-01, -1.60927066e-04],
           [-2.64566804e-01, -2.42771666e-01,  1.55786797e-02],
           [-2.51138687e-01, -2.23455574e-01,  1.11066195e-02]],
          shape=(1000000, 3)), 'axyz_ext': array([[ 0.29974924,  0.25034588,  0.03746345],
           [ 0.29432334,  0.25325348,  0.04561565],
           [ 0.30052444,  0.25662384,  0.04606712],
           ...,
           [-0.30365396, -0.25614964, -0.040934  ],
           [-0.30590711, -0.25372766, -0.05022086],
           [-0.30459965, -0.25094772, -0.03831283]], shape=(1000000, 3)), 'hpart': array([0.20771971, 0.24976131, 0.20235232, ..., 0.22745052, 0.30584142,
           0.22650342], shape=(1000000,)), 'uint': array([ 3.40709038e-09,  9.06719995e-07,  6.33136482e-07, ...,
           -4.58527033e-07, -2.14375395e-07, -4.80424414e-07],
          shape=(1000000,)), 'duint': array([ 1.75537810e-06,  8.98428441e-04,  6.23714789e-04, ...,
           -4.59495922e-04, -2.15374360e-04, -4.81095692e-04],
          shape=(1000000,))}




.. GENERATED FROM PYTHON SOURCE LINES 226-227

Performing a timestep loop

.. GENERATED FROM PYTHON SOURCE LINES 227-239

.. code-block:: Python
   :lineno-start: 227

    dt_stop = 0.001
    for i in range(10):
        t_target = i * dt_stop
        # skip if the model is already past the target
        if model.get_time() > t_target:
            continue

        model.evolve_until(i * dt_stop)

        # Dump name is "dump_xxxx.sham" where xxxx is the timestep
        model.dump(dump_folder + f"/dump_{i:04}.sham")





.. rst-class:: sphx-glr-script-out

 .. code-block:: none

    Info: iteration since start : 3                                                       [SPH][rank=0] 
    Info: time since start : 77.67301225300001 (s)                                        [SPH][rank=0] 
    Info: Dumping state to _to_trash/dump_0001.sham                                       [SPH][rank=0] 
    Info: dump to _to_trash/dump_0001.sham                                      [Shamrock Dump][rank=0]
                  - took 131.00 ms, bandwidth = 873.66 MB/s  
    ---------------- t = 0.001, dt = 0.001 ---------------- 
    Info: summary :                                                               [LoadBalance][rank=0] 
    Info:  - strategy "psweep" : max = 1000000 min = 1000000                      [LoadBalance][rank=0] 
    Info:  - strategy "round robin" : max = 1000000 min = 1000000                 [LoadBalance][rank=0] 
    Info: Loadbalance stats :                                                     [LoadBalance][rank=0]
        npatch = 1
        min = 1000000
        max = 1000000
        avg = 1000000
        efficiency = 100.00%  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 10.87 us   (1.9%)
       patch tree reduce : 1233.00 ns (0.2%)
       gen split merge   : 731.00 ns  (0.1%)
       split / merge op  : 0/0
       apply split merge : 1092.00 ns (0.2%)
       LB compute        : 533.89 us  (91.8%)
       LB move op cnt    : 0
       LB apply          : 4.17 us    (0.7%)  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 7.69 us    (76.0%)  
    Info: free boundaries skipping geometry update                            [PositionUpdated][rank=0] 
    Info: smoothing length iteration converged                                [Smoothinglength][rank=0]
      eps min = 3.59756982210602e-14, max = 9.999919608723482e-07
      iterations = 2  
    Info: conservation infos :                                                     [sph::Model][rank=0]
        sum v = (9.883777269165263e-06,-3.3104468402850225e-05,1.1568617173455943e-09)
        sum a = (3.4728350837426314e-19,-3.781155076543197e-18,8.216219588366713e-20)
        sum e = 0.05004074705060449
        sum de = 8.502063120608242e-06  
    Info: cfl dt = 0.002784776400983339 cfl multiplier : 0.7066666666666667        [sph::Model][rank=0] 
    Info: processing rate infos :                                                  [sph::Model][rank=0]
    ---------------------------------------------------------------------------------------
    | rank |  rate  (N.s^-1)  |     Nobj    | t compute (s) |  MPI   | alloc |  mem (max) |
    ---------------------------------------------------------------------------------------
    | 0    |    1.6529e+05    |     1000000 |   6.050e+00   |    0 % |   0 % |  950.83 MB |
    ---------------------------------------------------------------------------------------  
    Info: estimated rate : 0.5950421155396135 (tsim/hr)                            [sph::Model][rank=0] 
    Info: iteration since start : 4                                                       [SPH][rank=0] 
    Info: time since start : 83.87218156 (s)                                              [SPH][rank=0] 
    Info: Dumping state to _to_trash/dump_0002.sham                                       [SPH][rank=0] 
    Info: dump to _to_trash/dump_0002.sham                                      [Shamrock Dump][rank=0]
                  - took 127.94 ms, bandwidth = 894.55 MB/s  
    ---------------- t = 0.002, dt = 0.001 ---------------- 
    Info: summary :                                                               [LoadBalance][rank=0] 
    Info:  - strategy "psweep" : max = 1000000 min = 1000000                      [LoadBalance][rank=0] 
    Info:  - strategy "round robin" : max = 1000000 min = 1000000                 [LoadBalance][rank=0] 
    Info: Loadbalance stats :                                                     [LoadBalance][rank=0]
        npatch = 1
        min = 1000000
        max = 1000000
        avg = 1000000
        efficiency = 100.00%  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 7.16 us    (0.6%)
       patch tree reduce : 1192.00 ns (0.1%)
       gen split merge   : 742.00 ns  (0.1%)
       split / merge op  : 0/0
       apply split merge : 1112.00 ns (0.1%)
       LB compute        : 1164.78 us (98.3%)
       LB move op cnt    : 0
       LB apply          : 3.78 us    (0.3%)  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 2.56 us    (72.3%)  
    Info: free boundaries skipping geometry update                            [PositionUpdated][rank=0] 
    Info: smoothing length iteration converged                                [Smoothinglength][rank=0]
      eps min = 3.928401332424843e-14, max = 9.999915952970984e-07
      iterations = 2  
    Info: conservation infos :                                                     [sph::Model][rank=0]
        sum v = (9.883580481113704e-06,-3.3104548693157906e-05,1.1568571334709395e-09)
        sum a = (-2.97816784254612e-18,-9.351243737687476e-19,-1.6517142471458857e-20)
        sum e = 0.05004075760279957
        sum de = 1.275450403348239e-05  
    Info: cfl dt = 0.003193940312031096 cfl multiplier : 0.8044444444444444        [sph::Model][rank=0] 
    Info: processing rate infos :                                                  [sph::Model][rank=0]
    ---------------------------------------------------------------------------------------
    | rank |  rate  (N.s^-1)  |     Nobj    | t compute (s) |  MPI   | alloc |  mem (max) |
    ---------------------------------------------------------------------------------------
    | 0    |    1.6933e+05    |     1000000 |   5.906e+00   |    0 % |   0 % |  950.83 MB |
    ---------------------------------------------------------------------------------------  
    Info: estimated rate : 0.6095940679946079 (tsim/hr)                            [sph::Model][rank=0] 
    Info: iteration since start : 5                                                       [SPH][rank=0] 
    Info: time since start : 89.92534139300001 (s)                                        [SPH][rank=0] 
    Info: Dumping state to _to_trash/dump_0003.sham                                       [SPH][rank=0] 
    Info: dump to _to_trash/dump_0003.sham                                      [Shamrock Dump][rank=0]
                  - took 126.53 ms, bandwidth = 904.53 MB/s  
    ---------------- t = 0.003, dt = 0.001 ---------------- 
    Info: summary :                                                               [LoadBalance][rank=0] 
    Info:  - strategy "psweep" : max = 1000000 min = 1000000                      [LoadBalance][rank=0] 
    Info:  - strategy "round robin" : max = 1000000 min = 1000000                 [LoadBalance][rank=0] 
    Info: Loadbalance stats :                                                     [LoadBalance][rank=0]
        npatch = 1
        min = 1000000
        max = 1000000
        avg = 1000000
        efficiency = 100.00%  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 7.67 us    (1.8%)
       patch tree reduce : 1272.00 ns (0.3%)
       gen split merge   : 1002.00 ns (0.2%)
       split / merge op  : 0/0
       apply split merge : 972.00 ns  (0.2%)
       LB compute        : 401.24 us  (95.1%)
       LB move op cnt    : 0
       LB apply          : 3.85 us    (0.9%)  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 2.58 us    (68.6%)  
    Info: free boundaries skipping geometry update                            [PositionUpdated][rank=0] 
    Info: smoothing length iteration converged                                [Smoothinglength][rank=0]
      eps min = 4.314372909994344e-14, max = 9.999938675557479e-07
      iterations = 2  
    Info: conservation infos :                                                     [sph::Model][rank=0]
        sum v = (9.883384425660589e-06,-3.310462979107857e-05,1.1568495020018086e-09)
        sum a = (-1.5670109524204556e-18,-2.2090619264392153e-18,2.541098841762901e-20)
        sum e = 0.050040772405409085
        sum de = 1.700734113447608e-05  
    Info: cfl dt = 0.003486138014016258 cfl multiplier : 0.8696296296296296        [sph::Model][rank=0] 
    Info: processing rate infos :                                                  [sph::Model][rank=0]
    ---------------------------------------------------------------------------------------
    | rank |  rate  (N.s^-1)  |     Nobj    | t compute (s) |  MPI   | alloc |  mem (max) |
    ---------------------------------------------------------------------------------------
    | 0    |    1.6902e+05    |     1000000 |   5.916e+00   |    0 % |   0 % |  950.83 MB |
    ---------------------------------------------------------------------------------------  
    Info: estimated rate : 0.6084800530523211 (tsim/hr)                            [sph::Model][rank=0] 
    Info: iteration since start : 6                                                       [SPH][rank=0] 
    Info: time since start : 95.98569488400001 (s)                                        [SPH][rank=0] 
    Info: Dumping state to _to_trash/dump_0004.sham                                       [SPH][rank=0] 
    Info: dump to _to_trash/dump_0004.sham                                      [Shamrock Dump][rank=0]
                  - took 126.21 ms, bandwidth = 906.81 MB/s  
    ---------------- t = 0.004, dt = 0.001 ---------------- 
    Info: summary :                                                               [LoadBalance][rank=0] 
    Info:  - strategy "psweep" : max = 1000000 min = 1000000                      [LoadBalance][rank=0] 
    Info:  - strategy "round robin" : max = 1000000 min = 1000000                 [LoadBalance][rank=0] 
    Info: Loadbalance stats :                                                     [LoadBalance][rank=0]
        npatch = 1
        min = 1000000
        max = 1000000
        avg = 1000000
        efficiency = 100.00%  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 6.87 us    (1.6%)
       patch tree reduce : 1082.00 ns (0.3%)
       gen split merge   : 712.00 ns  (0.2%)
       split / merge op  : 0/0
       apply split merge : 942.00 ns  (0.2%)
       LB compute        : 404.87 us  (95.3%)
       LB move op cnt    : 0
       LB apply          : 3.82 us    (0.9%)  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 2.36 us    (70.0%)  
    Info: free boundaries skipping geometry update                            [PositionUpdated][rank=0] 
    Info: smoothing length iteration converged                                [Smoothinglength][rank=0]
      eps min = 4.6865656022720254e-14, max = 9.999306744900812e-07
      iterations = 2  
    Info: conservation infos :                                                     [sph::Model][rank=0]
        sum v = (9.883189105990373e-06,-3.3104711691901545e-05,1.1568388248212036e-09)
        sum a = (-2.715587628897287e-18,-3.0493186101154812e-18,-8.512681119905718e-20)
        sum e = 0.0500407914555922
        sum de = 2.1260579794904812e-05  
    Info: cfl dt = 0.003702564278479169 cfl multiplier : 0.9130864197530864        [sph::Model][rank=0] 
    Info: processing rate infos :                                                  [sph::Model][rank=0]
    ---------------------------------------------------------------------------------------
    | rank |  rate  (N.s^-1)  |     Nobj    | t compute (s) |  MPI   | alloc |  mem (max) |
    ---------------------------------------------------------------------------------------
    | 0    |    1.6788e+05    |     1000000 |   5.957e+00   |    0 % |   0 % |  950.83 MB |
    ---------------------------------------------------------------------------------------  
    Info: estimated rate : 0.6043811485707798 (tsim/hr)                            [sph::Model][rank=0] 
    Info: iteration since start : 7                                                       [SPH][rank=0] 
    Info: time since start : 102.087305794 (s)                                            [SPH][rank=0] 
    Info: Dumping state to _to_trash/dump_0005.sham                                       [SPH][rank=0] 
    Info: dump to _to_trash/dump_0005.sham                                      [Shamrock Dump][rank=0]
                  - took 128.50 ms, bandwidth = 890.60 MB/s  
    ---------------- t = 0.005, dt = 0.001 ---------------- 
    Info: summary :                                                               [LoadBalance][rank=0] 
    Info:  - strategy "psweep" : max = 1000000 min = 1000000                      [LoadBalance][rank=0] 
    Info:  - strategy "round robin" : max = 1000000 min = 1000000                 [LoadBalance][rank=0] 
    Info: Loadbalance stats :                                                     [LoadBalance][rank=0]
        npatch = 1
        min = 1000000
        max = 1000000
        avg = 1000000
        efficiency = 100.00%  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 7.15 us    (1.7%)
       patch tree reduce : 1102.00 ns (0.3%)
       gen split merge   : 682.00 ns  (0.2%)
       split / merge op  : 0/0
       apply split merge : 1092.00 ns (0.3%)
       LB compute        : 389.46 us  (95.0%)
       LB move op cnt    : 0
       LB apply          : 3.58 us    (0.9%)  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 2.58 us    (69.1%)  
    Info: free boundaries skipping geometry update                            [PositionUpdated][rank=0] 
    Info: smoothing length iteration converged                                [Smoothinglength][rank=0]
      eps min = 5.169036272632375e-14, max = 9.999633369052757e-07
      iterations = 2  
    Info: conservation infos :                                                     [sph::Model][rank=0]
        sum v = (9.882994525255597e-06,-3.310479439090972e-05,1.1568250982440996e-09)
        sum a = (-3.4558944247975454e-19,-1.4230153513872246e-18,-2.541098841762901e-20)
        sum e = 0.050040814752549914
        sum de = 2.551421853648265e-05  
    Info: cfl dt = 0.0038700311671878286 cfl multiplier : 0.9420576131687243       [sph::Model][rank=0] 
    Info: processing rate infos :                                                  [sph::Model][rank=0]
    ---------------------------------------------------------------------------------------
    | rank |  rate  (N.s^-1)  |     Nobj    | t compute (s) |  MPI   | alloc |  mem (max) |
    ---------------------------------------------------------------------------------------
    | 0    |    1.6950e+05    |     1000000 |   5.900e+00   |    0 % |   0 % |  950.83 MB |
    ---------------------------------------------------------------------------------------  
    Info: estimated rate : 0.6102132864295647 (tsim/hr)                            [sph::Model][rank=0] 
    Info: iteration since start : 8                                                       [SPH][rank=0] 
    Info: time since start : 108.13309212 (s)                                             [SPH][rank=0] 
    Info: Dumping state to _to_trash/dump_0006.sham                                       [SPH][rank=0] 
    Info: dump to _to_trash/dump_0006.sham                                      [Shamrock Dump][rank=0]
                  - took 127.15 ms, bandwidth = 900.08 MB/s  
    ---------------- t = 0.006, dt = 0.001 ---------------- 
    Info: summary :                                                               [LoadBalance][rank=0] 
    Info:  - strategy "psweep" : max = 1000000 min = 1000000                      [LoadBalance][rank=0] 
    Info:  - strategy "round robin" : max = 1000000 min = 1000000                 [LoadBalance][rank=0] 
    Info: Loadbalance stats :                                                     [LoadBalance][rank=0]
        npatch = 1
        min = 1000000
        max = 1000000
        avg = 1000000
        efficiency = 100.00%  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 7.18 us    (1.6%)
       patch tree reduce : 1053.00 ns (0.2%)
       gen split merge   : 661.00 ns  (0.1%)
       split / merge op  : 0/0
       apply split merge : 962.00 ns  (0.2%)
       LB compute        : 438.39 us  (95.6%)
       LB move op cnt    : 0
       LB apply          : 3.70 us    (0.8%)  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 2.62 us    (72.3%)  
    Info: free boundaries skipping geometry update                            [PositionUpdated][rank=0] 
    Info: smoothing length iteration converged                                [Smoothinglength][rank=0]
      eps min = 5.118853328034394e-14, max = 9.9998416943092e-07
      iterations = 3  
    Info: conservation infos :                                                     [sph::Model][rank=0]
        sum v = (9.882800686588224e-06,-3.310487788339004e-05,1.1568083141664227e-09)
        sum a = (-5.793705359219414e-18,2.642742795433417e-18,-1.1434944787933055e-20)
        sum e = 0.05004084229606356
        sum de = 2.9768252145916814e-05  
    Info: cfl dt = 0.004005966807028481 cfl multiplier : 0.9613717421124829        [sph::Model][rank=0] 
    Info: processing rate infos :                                                  [sph::Model][rank=0]
    ---------------------------------------------------------------------------------------
    | rank |  rate  (N.s^-1)  |     Nobj    | t compute (s) |  MPI   | alloc |  mem (max) |
    ---------------------------------------------------------------------------------------
    | 0    |    1.6706e+05    |     1000000 |   5.986e+00   |    0 % |   0 % |  950.83 MB |
    ---------------------------------------------------------------------------------------  
    Info: estimated rate : 0.6014166961671301 (tsim/hr)                            [sph::Model][rank=0] 
    Info: iteration since start : 9                                                       [SPH][rank=0] 
    Info: time since start : 114.26661741400001 (s)                                       [SPH][rank=0] 
    Info: Dumping state to _to_trash/dump_0007.sham                                       [SPH][rank=0] 
    Info: dump to _to_trash/dump_0007.sham                                      [Shamrock Dump][rank=0]
                  - took 132.19 ms, bandwidth = 865.80 MB/s  
    ---------------- t = 0.007, dt = 0.001 ---------------- 
    Info: summary :                                                               [LoadBalance][rank=0] 
    Info:  - strategy "psweep" : max = 1000000 min = 1000000                      [LoadBalance][rank=0] 
    Info:  - strategy "round robin" : max = 1000000 min = 1000000                 [LoadBalance][rank=0] 
    Info: Loadbalance stats :                                                     [LoadBalance][rank=0]
        npatch = 1
        min = 1000000
        max = 1000000
        avg = 1000000
        efficiency = 100.00%  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 7.32 us    (1.5%)
       patch tree reduce : 1232.00 ns (0.2%)
       gen split merge   : 711.00 ns  (0.1%)
       split / merge op  : 0/0
       apply split merge : 1062.00 ns (0.2%)
       LB compute        : 480.09 us  (95.8%)
       LB move op cnt    : 0
       LB apply          : 3.84 us    (0.8%)  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 2.88 us    (70.1%)  
    Info: free boundaries skipping geometry update                            [PositionUpdated][rank=0] 
    Info: smoothing length iteration converged                                [Smoothinglength][rank=0]
      eps min = 6.175350975866222e-14, max = 9.999783138459902e-07
      iterations = 3  
    Info: conservation infos :                                                     [sph::Model][rank=0]
        sum v = (9.882607593081656e-06,-3.310496216459064e-05,1.1567884609152483e-09)
        sum a = (1.8295911660692887e-19,3.496552006265752e-18,8.512681119905718e-20)
        sum e = 0.050040874086274195
        sum de = 3.40226702527547e-05  
    Info: cfl dt = 0.004116601548113821 cfl multiplier : 0.9742478280749886        [sph::Model][rank=0] 
    Info: processing rate infos :                                                  [sph::Model][rank=0]
    ---------------------------------------------------------------------------------------
    | rank |  rate  (N.s^-1)  |     Nobj    | t compute (s) |  MPI   | alloc |  mem (max) |
    ---------------------------------------------------------------------------------------
    | 0    |    1.6683e+05    |     1000000 |   5.994e+00   |    0 % |   0 % |  950.83 MB |
    ---------------------------------------------------------------------------------------  
    Info: estimated rate : 0.6005908483639171 (tsim/hr)                            [sph::Model][rank=0] 
    Info: iteration since start : 10                                                      [SPH][rank=0] 
    Info: time since start : 120.414583566 (s)                                            [SPH][rank=0] 
    Info: Dumping state to _to_trash/dump_0008.sham                                       [SPH][rank=0] 
    Info: dump to _to_trash/dump_0008.sham                                      [Shamrock Dump][rank=0]
                  - took 126.49 ms, bandwidth = 904.76 MB/s  
    ---------------- t = 0.008, dt = 0.0010000000000000009 ---------------- 
    Info: summary :                                                               [LoadBalance][rank=0] 
    Info:  - strategy "psweep" : max = 1000000 min = 1000000                      [LoadBalance][rank=0] 
    Info:  - strategy "round robin" : max = 1000000 min = 1000000                 [LoadBalance][rank=0] 
    Info: Loadbalance stats :                                                     [LoadBalance][rank=0]
        npatch = 1
        min = 1000000
        max = 1000000
        avg = 1000000
        efficiency = 100.00%  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 7.44 us    (1.8%)
       patch tree reduce : 1102.00 ns (0.3%)
       gen split merge   : 671.00 ns  (0.2%)
       split / merge op  : 0/0
       apply split merge : 972.00 ns  (0.2%)
       LB compute        : 397.78 us  (95.2%)
       LB move op cnt    : 0
       LB apply          : 3.67 us    (0.9%)  
    Info: Scheduler step timings :                                                  [Scheduler][rank=0]
       metadata sync     : 2.28 us    (72.4%)  
    Info: free boundaries skipping geometry update                            [PositionUpdated][rank=0] 
    Info: smoothing length iteration converged                                [Smoothinglength][rank=0]
      eps min = 6.76811789489175e-14, max = 9.999720767879178e-07
      iterations = 3  
    Info: conservation infos :                                                     [sph::Model][rank=0]
        sum v = (9.882415247814464e-06,-3.310504722978855e-05,1.1567655234357156e-09)
        sum a = (-7.758821796849391e-19,-1.2739375526704677e-18,-1.5246593050577406e-20)
        sum e = 0.05004091012360171
        sum de = 3.827745882709571e-05  
    Info: cfl dt = 0.003921166513495738 cfl multiplier : 0.9828318853833258        [sph::Model][rank=0] 
    Info: processing rate infos :                                                  [sph::Model][rank=0]
    ---------------------------------------------------------------------------------------
    | rank |  rate  (N.s^-1)  |     Nobj    | t compute (s) |  MPI   | alloc |  mem (max) |
    ---------------------------------------------------------------------------------------
    | 0    |    1.6565e+05    |     1000000 |   6.037e+00   |    0 % |   0 % |  950.83 MB |
    ---------------------------------------------------------------------------------------  
    Info: estimated rate : 0.5963378794415541 (tsim/hr)                            [sph::Model][rank=0] 
    Info: iteration since start : 11                                                      [SPH][rank=0] 
    Info: time since start : 126.59802511400001 (s)                                       [SPH][rank=0] 
    Info: Dumping state to _to_trash/dump_0009.sham                                       [SPH][rank=0] 
    Info: dump to _to_trash/dump_0009.sham                                      [Shamrock Dump][rank=0]
                  - took 140.50 ms, bandwidth = 814.56 MB/s  




.. GENERATED FROM PYTHON SOURCE LINES 240-241

Plot column integrated density

.. GENERATED FROM PYTHON SOURCE LINES 241-279

.. code-block:: Python
   :lineno-start: 241

    import matplotlib.pyplot as plt

    pixel_x = 1200
    pixel_y = 1080
    radius = 5
    center = (0.0, 0.0, 0.0)

    aspect = pixel_x / pixel_y
    pic_range = [-radius * aspect, radius * aspect, -radius, radius]
    delta_x = (radius * 2 * aspect, 0.0, 0.0)
    delta_y = (0.0, radius * 2, 0.0)

    arr_rho = model.render_cartesian_column_integ(
        "rho", "f64", center=(0.0, 0.0, 0.0), delta_x=delta_x, delta_y=delta_y, nx=pixel_x, ny=pixel_y
    )

    import copy

    import matplotlib

    my_cmap = copy.copy(matplotlib.colormaps.get_cmap("gist_heat"))  # copy the default cmap
    my_cmap.set_bad(color="black")

    fig_width = 6
    fig_height = fig_width / aspect
    plt.figure(figsize=(fig_width, fig_height))
    res = plt.imshow(arr_rho, cmap=my_cmap, origin="lower", extent=pic_range, norm="log", vmin=1e-9)

    cbar = plt.colorbar(res, extend="both")
    cbar.set_label(r"$\int \rho \, \mathrm{d} z$ [code unit]")
    # or r"$\rho$ [code unit]" for slices

    plt.title("t = {:0.3f} [code unit]".format(model.get_time()))
    plt.xlabel("x")
    plt.ylabel("z")
    plt.show()





.. image-sg:: /_as_gen/sph/images/sphx_glr_run_sph_basic_disc_001.png
   :alt: t = 0.009 [code unit]
   :srcset: /_as_gen/sph/images/sphx_glr_run_sph_basic_disc_001.png
   :class: sphx-glr-single-img


.. rst-class:: sphx-glr-script-out

 .. code-block:: none

    Info: compute_column_integ field_name: rho, center: (0,0,0), delta_x: (11.11111111111111,0,0), delta_y: (0,10,0), nx: 1200, ny: 1080  [sph::CartesianRender][rank=0] 
    Info: compute_column_integ took 9.47 s                               [sph::CartesianRender][rank=0] 




.. GENERATED FROM PYTHON SOURCE LINES 280-281

Plot vertical profiles at r=1

.. GENERATED FROM PYTHON SOURCE LINES 281-356

.. code-block:: Python
   :lineno-start: 281

    import numpy as np

    dat = ctx.collect_data()

    for rcenter in [1.0, 2.0, 3.0]:

        z = []
        h = []
        vz = []
        az = []

        delta_r = 0.01

        for i in range(len(dat["xyz"])):
            r = (dat["xyz"][i][0] ** 2 + dat["xyz"][i][1] ** 2) ** 0.5
            if r < rcenter + delta_r and r > rcenter - delta_r:
                z.append(dat["xyz"][i][2])
                h.append(dat["hpart"][i])
                vz.append(dat["vxyz"][i][2])
                az.append(dat["axyz"][i][2])

        rho = pmass * (model.get_hfact() / np.array(h)) ** 3

        fig, axs = plt.subplots(nrows=3, ncols=1, sharex=True)

        from scipy.optimize import curve_fit

        def func(x, a, c):
            return a * np.exp(-((x / c) ** 2) / 2)

        rho_0 = 0.001
        p0 = [rho_0, H_profile(rcenter)]  # a, b, c
        popt, pcov = curve_fit(func, z, rho, p0=p0)

        z_ana = np.linspace(-5.0 * H_profile(rcenter), 5.0 * H_profile(rcenter), 100)
        rho_fit = func(z_ana, *popt)

        axs[0].scatter(z, rho, label="rho")

        axs[0].plot(z_ana, rho_fit, c="black", label="gaussian fit")
        stddev = abs(popt[1])
        axs[0].annotate(
            f"Stddev: {stddev:.5f}",
            xy=(0.05, 0.95),
            xycoords="axes fraction",
            fontsize=10,
            verticalalignment="top",
            bbox=dict(boxstyle="round", fc="w"),
        )

        axs[0].set_ylabel("rho")
        axs[0].legend()

        axs[1].scatter(z, vz, label="vz")

        vz_fit = np.polyfit(z, vz, 1)
        vz_fit_fn = np.poly1d(vz_fit)
        axs[1].plot(z_ana, vz_fit_fn(z_ana), c="red", label="linear fit")

        axs[1].set_ylabel("vz")
        axs[1].legend()

        axs[2].scatter(z, az, label="az")

        az_fit = np.polyfit(z, az, 1)
        az_fit_fn = np.poly1d(az_fit)
        print(f"r={rcenter} az_fit={az_fit}")
        axs[2].plot(z_ana, az_fit_fn(z_ana), c="red", label="linear fit")

        axs[2].set_ylabel("az")
        axs[2].set_xlabel("z")
        axs[2].legend()

        plt.tight_layout()
        plt.show()



.. rst-class:: sphx-glr-horizontal


    *

      .. image-sg:: /_as_gen/sph/images/sphx_glr_run_sph_basic_disc_002.png
         :alt: run sph basic disc
         :srcset: /_as_gen/sph/images/sphx_glr_run_sph_basic_disc_002.png
         :class: sphx-glr-single-img

    *

      .. image-sg:: /_as_gen/sph/images/sphx_glr_run_sph_basic_disc_003.png
         :alt: run sph basic disc
         :srcset: /_as_gen/sph/images/sphx_glr_run_sph_basic_disc_003.png
         :class: sphx-glr-single-img

    *

      .. image-sg:: /_as_gen/sph/images/sphx_glr_run_sph_basic_disc_004.png
         :alt: run sph basic disc
         :srcset: /_as_gen/sph/images/sphx_glr_run_sph_basic_disc_004.png
         :class: sphx-glr-single-img


.. rst-class:: sphx-glr-script-out

 .. code-block:: none

    adding ->  xyz
    adding ->  vxyz
    adding ->  axyz
    adding ->  axyz_ext
    adding ->  hpart
    adding ->  uint
    adding ->  duint
    r=1.0 az_fit=[ 2.6665086 -0.0194775]
    r=2.0 az_fit=[ 0.14429902 -0.00043202]
    r=3.0 az_fit=[0.01686233 0.00431511]





.. rst-class:: sphx-glr-timing

   **Total running time of the script:** (1 minutes 52.824 seconds)

**Estimated memory usage:**  1086 MB


.. _sphx_glr_download__as_gen_sph_run_sph_basic_disc.py:

.. only:: html

  .. container:: sphx-glr-footer sphx-glr-footer-example

    .. container:: sphx-glr-download sphx-glr-download-jupyter

      :download:`Download Jupyter notebook: run_sph_basic_disc.ipynb <run_sph_basic_disc.ipynb>`

    .. container:: sphx-glr-download sphx-glr-download-python

      :download:`Download Python source code: run_sph_basic_disc.py <run_sph_basic_disc.py>`

    .. container:: sphx-glr-download sphx-glr-download-zip

      :download:`Download zipped: run_sph_basic_disc.zip <run_sph_basic_disc.zip>`


.. only:: html

 .. rst-class:: sphx-glr-signature

    `Gallery generated by Sphinx-Gallery <https://sphinx-gallery.github.io>`_
